<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use Illuminate\Http\Request;
use App\Services\AnnouncementService;
use App\Models\Announcement;
use App\Http\Requests\Backend\AnnouncementRequest;
use App\Exceptions\GeneralException;
use Illuminate\Support\Facades\Log;
use Exception;
use Throwable;

class AnnouncementController extends Controller implements StatusInterface
{
    /**
     * @var AnnouncementService
     */
    private $announcementService;


    /**
     * AnnouncementController constructor.
     *
     * @param AnnouncementService  $announcementService
     *
     */
    public function __construct(AnnouncementService $announcementService)
    {
        $this->announcementService = $announcementService;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
         //Obtener los selects
        $announcements = Announcement::orderBy('enabled', 'desc')->paginate(10);
        return view('backend.pages.announcement.index', ['announcements' => $announcements]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
         return view('backend.pages.announcement.form');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param AnnouncementRequest $request
     * @return \Illuminate\Http\Response
     * @throws \App\Exceptions\GeneralException
     * @throws \Throwable
     */
    public function store(AnnouncementRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $announcement = $this->announcementService->createAnnouncement($request);
                if ($announcement){
                    return redirect()->route('admin.auth.announcement.announcement.index')->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $announcement = Announcement::where('id', $id)->first();
          if ($announcement) {
            return view('backend.pages.announcement.form', ['announcement' => $announcement]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

 /**
     * Update the specified resource in storage.
     *
     * @param AnnouncementRequest $request
     * @return \Illuminate\Http\Response
     * @throws \App\Exceptions\GeneralException
     * @throws \Throwable
     */
    public function update(AnnouncementRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {

                $announcement = $this->announcementService->updateAnnouncement($request);
                if ($announcement){
                    return redirect()->route('admin.auth.announcement.announcement.index')->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.announcement.announcement.index')->withErrors(__('auth.general_error'))->withInput();
    }

     /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete($id)
    {
        //
        $data = self::KO;
        $announcement = Announcement::where('id', $id)->first();
        if ($announcement) {
            if ($this->announcementService->delete($announcement)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
         //
         $data = self::KO;
         $announcement = Announcement::where('id', $id)->first();
         if ($announcement) {
             if ($this->announcementService->destroy($announcement)) {
                 $data = ['success' => self::OK];
             }
         }
         return response()->json(json_encode($data, true));
    }
}
