<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use App\Services\HelperService;
use Illuminate\Support\Collection;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;
use App\Services\CampaignService;
use App\Http\Requests\Backend\CampaignRequest;
use App\Models\Campaign;
use App\Models\Company;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Throwable;

class CampaignController extends Controller implements StatusInterface
{
     /**
     * @var campaignService
     */
    private $campaignService;

    /**
     * HourController constructor.
     *
     * @param CampaignService $campaignService
     */
    public function __construct(CampaignService $campaignService)
    {
        $this->campaignService = $campaignService;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('backend.pages.campaign.index');
    }

     /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function jsonList(Request $request)
    {
        if($request->ajax()){
            $data = new Collection;

            $campaigns = Campaign::all();
            foreach($campaigns as $campaign) {
                $company = isset($campaign->myCompany->name) ? $campaign->myCompany->name :  '';
                $data->push([
                    'id'         => $campaign->id,
                    'name'       => $campaign->name,
                    'slug'       => $campaign->slug,
                    'company'    => $company,
                    'active'     => $campaign->active,
                    'diff'       => Carbon::parse($campaign->created_at)->diffForHumans(),
                ]);
            }

            if ($campaigns) {
                return DataTables::of($data)
                ->addColumn('action', function($data){

                    $id = isset($data['id']) ? $data['id'] : -1;
                    $routeShow =  route('admin.auth.campaign.campaign.show', ['campaign' => $id]);
                    $routeEdit =  route('admin.auth.campaign.campaign.edit', ['campaign' => $id]);
                    $routeDelete = route('admin.auth.campaign.campaign.delete', ['campaign' => $id]);
                    $slug = isset($data['name']) ? $data['name'] : -1;
                    $buttons = '<a href="'.$routeShow.'" type="button" name="view" class="btn btn-info btn-sm view"><i class="fas fa-search"> '.trans('backend.button.view').'</i></a>
                    <a href="'.$routeEdit.'" type="button" name="edit" class="btn btn-primary btn-sm edit"><i class="fas fa-pencil-alt"> '.trans('backend.button.edit').'</i></a> ';
                    $buttons .= '<button attr-delete="'.$routeDelete.'" type="button" name="delete" attr-id="'.$id.'" attr-slug="'.$slug.'" class="btn btn-danger btn-sm delete"><i class="fas fa-trash"> '.trans('backend.button.delete').'</i></button>';
                    return $buttons;
                })
                ->addColumn('active-icon', function($data){
                    return HelperService::getYesNoIconTable($data['active']);
                })
                ->addColumn('company-icon', function($data){
                    if ($data['company']) {
                        $icon = '<span class="badge badge-pill badge-success" data-toggle="tooltip" title="'.trans('backend.icon.yes').'" data-original-title="'.$data['company'].'">'.$data['company'].'</span>';
                    } else {
                        $icon = '<span class="badge badge-pill badge-danger" data-toggle="tooltip" title="'.trans('backend.icon.no').'" data-original-title="'.trans('backend.icon.no').'">'.trans('backend.icon.no').'</span>';
                    }
                    return $icon;
                })
                ->rawColumns(['action' => 'active-icon', 'company' => 'company-icon'])
                ->make(true);
            }
        }
        return DataTables::of(new Collection);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {

        //Obtener los selects
        $companies = Company::all();
        $campaign = null;

        return view('backend.pages.campaign.form', ['campaign' => $campaign, 'companies' => $companies ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param CampaignRequest $request
     * @return \Illuminate\Http\Response
     * @throws GeneralException
     */
    public function store(CampaignRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $campaign = $this->campaignService->createCampaign($request);
                if ($campaign){
                    return redirect()->route('admin.auth.campaign.campaign.show', $campaign)->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.campaign.campaign.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $campaign = Campaign::where('id', $id)->first();
        if ($campaign) {
            return view('backend.pages.campaign.show', ['campaign' => $campaign]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //Obtener los selects
        $companies = Company::all();

        if ($companies->isEmpty()) {
            $companies = null;
        }

        $campaign = Campaign::where('id', $id)->first();
        if ($campaign) {
            return view('backend.pages.campaign.form', ['campaign' => $campaign, 'companies' => $companies]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * @param CampaignRequest $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws GeneralException
     * @throws ValidationException
     */
    public function update(CampaignRequest $request)
    {
          //
          $validator = null;
          try {
              if (isset($request->validator)) {
                  $validator = $request->validator;
              }
              if (!(isset($validator) && $validator->fails())) {

                  $campaign = $this->campaignService->updateCampaign($request);
                  if ($campaign){
                      return redirect()->route('admin.auth.campaign.campaign.show', $campaign)->withFlashSuccess(__('backend.alert.success'));
                  }
              }
          } catch (ValidationException $e) {
              throw new ValidationException($e->validator);
          } catch (Exception | Throwable $e) {
              Log::error($e);
              throw new GeneralException(__('auth.general_error'));
          }
        return redirect()->route('admin.auth.campaign.campaign.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * @param $campaignId
     * @return \Illuminate\Http\JsonResponse
     */
    public function delete($campaignId)
    {
        $data = self::KO;
        $campaign = Campaign::where('id', $campaignId)->first();

        if ($campaign) {
            if ($this->campaignService->delete($campaign)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $campaignId
     * @return \Illuminate\Http\Response
     */
    public function destroy($campaignId)
    {
        $data = self::KO;
        $campaign = Campaign::where('id', $campaignId)->first();

        if ($campaign && $campaign->id != 1) {
            if ($this->campaignService->destroy($campaign)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

        /**
     * Remove the specified resource from storage.
     *
     * @param $campaignId
     * @return \Illuminate\Http\Response
     */
    public function deleteAvatar($campaignId){
        $data = self::KO;
        $campaign = Campaign::where('id', $campaignId)->first();
        if ($campaign) {
            if ($this->campaignService->deleteImage($campaign)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }
}
