<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use App\Services\HelperService;
use Illuminate\Support\Collection;
use Illuminate\Http\Request;
use App\Http\Requests\Backend\CompanyRequest;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\Log;
use App\Services\CompanyService;
use App\Models\Company;
use App\Models\Via;
use App\Models\Province;
use App\Models\Country;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Throwable;

class CompanyController extends Controller implements StatusInterface
{
    /**
     * @var CompanyService
     */
    protected $companyService;


    /**
     * CompanyController constructor.
     *
     * @param  CompanyService  $companyService
     *
     */
    public function __construct(CompanyService $companyService)
    {
        $this->companyService = $companyService;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('backend.pages.company.index');
    }

    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function jsonList(Request $request)
    {
        if($request->ajax()){
            $data = new Collection;

            $companies = Company::all();
            foreach($companies as $company) {
                $data->push([
                    'id'         => $company->id,
                    'name'       => $company->name,
                    'slug'       => $company->slug,
                    'nif'        => $company->nif,
                    'email'      => $company->email,
                    'active'     => $company->active,
                    'diff'       => Carbon::parse($company->updated_at)->diffForHumans(),
                ]);
            }

            if ($companies) {
                return DataTables::of($data)
                ->addColumn('action', function($data){

                    $id = isset($data['id']) ? $data['id'] : -1;
                    $routeShow =  route('admin.auth.company.company.show', ['cia' => $id]);
                    $routeEdit =  route('admin.auth.company.company.edit', ['cia' => $id]);
                    $routeDelete = route('admin.auth.company.company.delete', ['cia' => $id]);
                    $slug = isset($data['slug']) ? $data['slug'] : -1;
                    return '<a href="'.$routeShow.'" type="button" name="view" class="btn btn-info btn-sm view"><i class="fas fa-search"> '.trans('backend.button.view').'</i></a>
                    <a href="'.$routeEdit.'" type="button" name="edit" class="btn btn-primary btn-sm edit"><i class="fas fa-pencil-alt"> '.trans('backend.button.edit').'</i></a>
                    <button attr-delete="'.$routeDelete.'" type="button" name="delete" attr-id="'.$id.'" attr-slug="'.$slug.'" class="btn btn-danger btn-sm delete"><i class="fas fa-trash"> '.trans('backend.button.delete').'</i></button>';
                })
                ->addColumn('active-icon', function($data){
                    return HelperService::getYesNoIconTable($data['active']);
                })
                ->rawColumns(['action', 'active-icon'])
                ->make(true);
            }
        }
        return DataTables::of(new Collection);
    }

    /**
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\Response
     */
    public function create()
    {
        //Obtener los selects
        $company = null;
        $vias = Via::all();
        $provinces = Province::all();
        $countries = Country::all();
        if ($vias->isEmpty()) {
            $vias = null;
        }

        if ($provinces->isEmpty()) {
            $provinces = null;
        }

        if ($countries->isEmpty()) {
            $countries = null;
        }
        $address = null;
        return view('backend.pages.company.form', ['company' => $company, 'address' => $address,'vias' => $vias, 'provinces' => $provinces, 'countries' => $countries]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param CompanyRequest $request
     * @return \Illuminate\Http\Response
     * @throws GeneralException
     */
    public function store(CompanyRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $company = $this->companyService->createCompany($request);
                if ($company){
                    return redirect()->route('admin.auth.company.company.show', $company)->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.company.company.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $company = Company::where('id', $id)->first();
        if ($company) {
            return view('backend.pages.company.show', ['company' => $company]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\Response
     */
    public function edit($id)
    {
        //Obtener los selects
        $vias = Via::all();
        $provinces = Province::all();
        $countries = Country::all();

        if ($vias->isEmpty()) {
            $vias = null;
        }

        if ($provinces->isEmpty()) {
            $provinces = null;
        }

        if ($countries->isEmpty()) {
            $countries = null;
        }

        $company = Company::where('id', $id)->first();
        if ($company) {
            return view('backend.pages.company.form', ['company' => $company, 'address' => $company->address, 'vias' => $vias, 'provinces' => $provinces, 'countries' => $countries]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * @param CompanyRequest $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws GeneralException
     * @throws ValidationException
     */
    public function update(CompanyRequest $request): \Illuminate\Http\RedirectResponse
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {

                $company = $this->companyService->updateCompany($request);
                if ($company){
                    return redirect()->route('admin.auth.company.company.show', $company)->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.company.company.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $companyId
     * @return \Illuminate\Http\Response
     */
    public function delete($companyId)
    {
        $data = self::KO;
        $company = Company::where('id', $companyId)->first();
        if ($company) {
            if ($this->companyService->delete($company)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $companyId
     * @return \Illuminate\Http\Response
     */
    public function destroy($companyId): \Illuminate\Http\Response
    {
        $data = self::KO;
        $company = Company::where('id', $companyId)->first();
        if ($company) {
            if ($this->companyService->destroy($company)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param $companyId
     * @return \Illuminate\Http\Response
     */
    public function deleteAvatar($companyId)
    {
        $data = self::KO;
        $company = Company::where('id', $companyId)->first();
        if ($company) {
            if ($this->companyService->deleteImage($company)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }
}
