<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use Illuminate\Support\Collection;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;
use App\Services\DelegationService;
use App\Http\Requests\Backend\DelegationRequest;
use App\Models\Delegation;
use App\Models\Company;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Throwable;

class DelegationController extends Controller implements StatusInterface
{
    /**
     * @var DelegationService
     */
    private $delegationService;

    /**
     * delegationController constructor.
     *
     * @param  delegationService  $delegationService
     *
     */
    public function __construct(delegationService $delegationService)
    {
        $this->delegationService = $delegationService;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('backend.pages.delegation.index');
    }

     /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function jsonList(Request $request)
    {
        if($request->ajax()){
            $data = new Collection;

            $delegations = Delegation::all();
            foreach($delegations as $delegation) {
                $data->push([
                    'id'         => $delegation->id,
                    'name'       => $delegation->name,
                    'slug'       => $delegation->slug,
                    'company'    => isset($delegation->myCompany->name) ? $delegation->myCompany->name :  false,
                    'active'     => $delegation->active,
                    'diff'       => Carbon::parse($delegation->created_at)->diffForHumans(),
                ]);
            }

            if ($delegations) {
                return DataTables::of($data)
                ->addColumn('action', function($data){

                    $id = isset($data['id']) ? $data['id'] : -1;
                    $routeShow =  route('admin.auth.delegation.delegation.show', ['delegation' => $id]);
                    $routeEdit =  route('admin.auth.delegation.delegation.edit', ['delegation' => $id]);
                    $routeDelete = route('admin.auth.delegation.delegation.delete', ['delegation' => $id]);
                    $slug = isset($data['name']) ? $data['name'] : -1;
                    $buttons = '<a href="'.$routeShow.'" type="button" name="view" class="btn btn-info btn-sm view"><i class="fas fa-search"> '.trans('backend.button.view').'</i></a>
                    <a href="'.$routeEdit.'" type="button" name="edit" class="btn btn-primary btn-sm edit"><i class="fas fa-pencil-alt"> '.trans('backend.button.edit').'</i></a> ';
                    $buttons .= '<button attr-delete="'.$routeDelete.'" type="button" name="delete" attr-id="'.$id.'" attr-slug="'.$slug.'" class="btn btn-danger btn-sm delete"><i class="fas fa-trash"> '.trans('backend.button.delete').'</i></button>';
                    return $buttons;
                })
                ->addColumn('active-icon', function($data){
                    if ($data['active']) {
                        $icon = '<span class="badge badge-pill badge-success" data-toggle="tooltip" title="'.trans('backend.icon.yes').'" data-original-title="'.trans('backend.icon.yes').'">'.trans('backend.icon.yes').'</span>';
                    } else {
                        $icon = '<span class="badge badge-pill badge-danger" data-toggle="tooltip" title="'.trans('backend.icon.no').'" data-original-title="'.trans('backend.icon.no').'">'.trans('backend.icon.no').'</span>';
                    }
                    return $icon;
                })
                ->addColumn('company-icon', function($data){
                    if ($data['company']) {
                        $icon = '<span class="badge badge-pill badge-success" data-toggle="tooltip" title="'.trans('backend.icon.yes').'" data-original-title="'.$data['company'].'">'.$data['company'].'</span>';
                    } else {
                        $icon = '<span class="badge badge-pill badge-danger" data-toggle="tooltip" title="'.trans('backend.icon.no').'" data-original-title="'.trans('backend.icon.no').'">'.trans('backend.icon.no').'</span>';
                    }
                    return $icon;
                })
                ->rawColumns(['action' => 'active-icon', 'company' => 'company-icon'])
                ->make(true);
            }
        }
        return DataTables::of(new Collection);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //Obtener los selects
        $companies = Company::all();
        $delegation = null;

        return view('backend.pages.delegation.form', ['companies' => $companies, 'delegation' => $delegation]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param DelegationRequest $request
     * @return \Illuminate\Http\Response
     * @throws GeneralException
     */
    public function store(DelegationRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $delegation = $this->delegationService->createDelegation($request);
                if ($delegation){
                    return redirect()->route('admin.auth.delegation.delegation.show', $delegation)->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.delegation.delegation.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $delegation = Delegation::where('id', $id)->first();
        if ($delegation) {
            return view('backend.pages.delegation.show', ['delegation' => $delegation]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
          //Obtener los selects
          $companies = Company::all();

          if ($companies->isEmpty()) {
             $companies = null;
         }

          $delegation = Delegation::where('id', $id)->first();
          if ($delegation) {
              return view('backend.pages.delegation.form', ['delegation' => $delegation, 'companies' => $companies]);
          } else {
              return redirect()->back()->withFlashDanger(__('auth.general_error'));
          }
    }

    /**
     * @param DelegationRequest $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws GeneralException
     * @throws ValidationException
     */
    public function update(DelegationRequest $request)
    {
          //
          $validator = null;
          try {
              if (isset($request->validator)) {
                  $validator = $request->validator;
              }
              if (!(isset($validator) && $validator->fails())) {

                  $delegation = $this->delegationService->updateDelegation($request);
                  if ($delegation){
                      return redirect()->route('admin.auth.delegation.delegation.show', $delegation)->withFlashSuccess(__('backend.alert.success'));
                  }
              }
          } catch (ValidationException $e) {
              throw new ValidationException($e->validator);
          } catch (Exception | Throwable $e) {
              Log::error($e);
              throw new GeneralException(__('auth.general_error'));
          }
        return redirect()->route('admin.auth.delegation.delegation.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * @param $delegationId
     * @return \Illuminate\Http\JsonResponse
     */
    public function delete($delegationId)
    {
        $data = self::KO;
        $delegation = Delegation::where('id', $delegationId)->first();

        if ($delegation) {
            if ($this->delegationService->delete($delegation)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $delegationId
     * @return \Illuminate\Http\Response
     */
    public function destroy($delegationId)
    {
        $data = self::KO;
        $delegation = Delegation::where('id', $delegationId)->first();

        if ($delegation && $delegation->id != 1) {
            if ($this->delegationService->destroy($delegation)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

        /**
     * Remove the specified resource from storage.
     *
     * @param $delegationId
     * @return \Illuminate\Http\Response
     */
    public function deleteAvatar($delegationId){
        $data = self::KO;
        $delegation = Delegation::where('id', $delegationId)->first();
        if ($delegation) {
            if ($this->delegationService->deleteImage($delegation)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }
}
