<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use Illuminate\Support\Collection;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;
use App\Services\OfficeService;
use App\Http\Requests\Backend\OfficeRequest;
use App\Models\Office;
use App\Models\Delegation;
use App\Services\HelperService;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Throwable;

class OfficeController extends Controller implements StatusInterface
{
    /**
     * @var OfficeService
     */
    private $officeService;

    /**
     * OfficeController constructor.
     *
     * @param  OfficeService  $officeService
     *
     */
    public function __construct(OfficeService $officeService)
    {
        $this->officeService = $officeService;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('backend.pages.office.index');
    }

     /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function jsonList(Request $request)
    {
        if($request->ajax()){
            $data = new Collection;

            $offices = Office::all();
            foreach($offices as $office) {
                $company = isset($office->myDelegation->myCompany->name) ? $office->myDelegation->myCompany->name.'->' :  '';
                $data->push([
                    'id'         => $office->id,
                    'name'       => $office->name,
                    'slug'       => $office->slug,
                    'delegation' => isset($office->myDelegation->name) ? $company.$office->myDelegation->name :  false,
                    'active'     => $office->active,
                    'diff'       => Carbon::parse($office->created_at)->diffForHumans(),
                ]);
            }

            if ($offices) {
                return DataTables::of($data)
                ->addColumn('action', function($data){

                    $id = isset($data['id']) ? $data['id'] : -1;
                    $routeShow =  route('admin.auth.office.office.show', ['office' => $id]);
                    $routeEdit =  route('admin.auth.office.office.edit', ['office' => $id]);
                    $routeDelete = route('admin.auth.office.office.delete', ['office' => $id]);
                    $slug = isset($data['name']) ? $data['name'] : -1;
                    $buttons = '<a href="'.$routeShow.'" type="button" name="view" class="btn btn-info btn-sm view"><i class="fas fa-search"> '.trans('backend.button.view').'</i></a>
                    <a href="'.$routeEdit.'" type="button" name="edit" class="btn btn-primary btn-sm edit"><i class="fas fa-pencil-alt"> '.trans('backend.button.edit').'</i></a> ';
                    $buttons .= '<button attr-delete="'.$routeDelete.'" type="button" name="delete" attr-id="'.$id.'" attr-slug="'.$slug.'" class="btn btn-danger btn-sm delete"><i class="fas fa-trash"> '.trans('backend.button.delete').'</i></button>';
                    return $buttons;
                })
                ->addColumn('active-icon', function($data){
                    return HelperService::getYesNoIconTable($data['active']);
                })
                ->addColumn('delegation-icon', function($data){
                    if ($data['delegation']) {
                        $icon = '<span class="badge badge-pill badge-success" data-toggle="tooltip" title="'.trans('backend.icon.yes').'" data-original-title="'.$data['delegation'].'">'.$data['delegation'].'</span>';
                    } else {
                        $icon = '<span class="badge badge-pill badge-danger" data-toggle="tooltip" title="'.trans('backend.icon.no').'" data-original-title="'.trans('backend.icon.no').'">'.trans('backend.icon.no').'</span>';
                    }
                    return $icon;
                })
                ->rawColumns(['action' => 'active-icon', 'delegation' => 'delegation-icon'])
                ->make(true);
            }
        }
        return DataTables::of(new Collection);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //Obtener los selects
        $delegations = Delegation::all();
        $office = null;

        return view('backend.pages.office.form', ['delegations' => $delegations, 'office' => $office]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param OfficeRequest $request
     * @return \Illuminate\Http\Response
     * @throws GeneralException
     */
    public function store(OfficeRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $office = $this->officeService->createOffice($request);
                if ($office){
                    return redirect()->route('admin.auth.office.office.show', $office)->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        } catch (Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.office.office.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $office = Office::where('id', $id)->first();
        if ($office) {
            return view('backend.pages.office.show', ['office' => $office]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
          //Obtener los selects
          $delegations = Delegation::all();

          if ($delegations->isEmpty()) {
             $delegations = null;
         }

          $office = Office::where('id', $id)->first();
          if ($office) {
              return view('backend.pages.office.form', ['office' => $office, 'delegations' => $delegations]);
          } else {
              return redirect()->back()->withFlashDanger(__('auth.general_error'));
          }
    }

    /**
     * @param OfficeRequest $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws GeneralException
     * @throws ValidationException
     */
    public function update(OfficeRequest $request)
    {
          //
          $validator = null;
          try {
              if (isset($request->validator)) {
                  $validator = $request->validator;
              }
              if (!(isset($validator) && $validator->fails())) {

                  $office = $this->officeService->updateOffice($request);
                  if ($office){
                      return redirect()->route('admin.auth.office.office.show', $office)->withFlashSuccess(__('backend.alert.success'));
                  }
              }
          } catch (ValidationException $e) {
              throw new ValidationException($e->validator);
          } catch (Exception $e) {
              Log::error($e);
              throw new GeneralException(__('auth.general_error'));
          } catch (Throwable $e) {
              Log::error($e);
              throw new GeneralException(__('auth.general_error'));
          }
          return redirect()->route('admin.auth.office.office.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * @param $officeId
     * @return \Illuminate\Http\JsonResponse
     */
    public function delete($officeId)
    {
        $data = self::KO;
        $office = Office::where('id', $officeId)->first();

        if ($office) {
            if ($this->officeService->delete($office)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $officeId
     * @return \Illuminate\Http\Response
     */
    public function destroy($officeId)
    {
        $data = self::KO;
        $office = Office::where('id', $officeId)->first();

        if ($office && $office->id != 1) {
            if ($this->officeService->destroy($office)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

        /**
     * Remove the specified resource from storage.
     *
     * @param $officeId
     * @return \Illuminate\Http\Response
     */
    public function deleteAvatar($officeId){
        $data = self::KO;
        $office = Office::where('id', $officeId)->first();
        if ($office) {
            if ($this->officeService->deleteImage($office)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }
}
