<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use App\Services\HelperService;
use Illuminate\Support\Collection;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;
use App\Services\TeamService;
use App\Services\MasterService;
use App\Http\Requests\Backend\TeamRequest;
use App\Models\Team;
use App\Models\Office;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use App\Http\Requests\Backend\MasterRequest;
use Throwable;

class TeamController extends Controller implements StatusInterface
{
    /**
     * @var TeamService
     */
    private $teamService;
    /**
     * @var MasterService
     */
    private $masterService;

    /**
     * TeamController constructor.
     *
     * @param  TeamService  $teamService
     *
     */
    public function __construct(TeamService $teamService, MasterService $masterService)
    {
        $this->teamService = $teamService;
        $this->masterService = $masterService;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $teams = Team::all();
        return view('backend.pages.team.index', ['teams' => $teams]);
    }

     /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function jsonList(Request $request)
    {
        if($request->ajax()){
            $data = new Collection;

            $teams = Team::all();
            foreach($teams as $team) {
                $office = isset($team->myOffice->name) ? $team->myOffice->name :  '';
                $delegation = isset($team->myOffice->myDelegation->name) ? $team->myOffice->myDelegation->name.'->' :  '';
                $company = isset($team->myOffice->myDelegation->name) ? $team->myOffice->myDelegation->name.'->' :  '';
                $data->push([
                    'id'         => $team->id,
                    'name'       => $team->name,
                    'slug'       => $team->slug,
                    'office'     => isset($team->myOffice->name) ?  $company.$delegation.$office  :  false,
                    'active'     => $team->active,
                    'diff'       => Carbon::parse($team->created_at)->diffForHumans(),
                ]);
            }

            if ($teams) {
                return DataTables::of($data)
                ->addColumn('action', function($data){

                    $id = isset($data['id']) ? $data['id'] : -1;
                    $routeShow =  route('admin.auth.team.team.show', ['team' => $id]);
                    $routeEdit =  route('admin.auth.team.team.edit', ['team' => $id]);
                    $routeDelete = route('admin.auth.team.team.delete', ['team' => $id]);
                    $slug = isset($data['name']) ? $data['name'] : -1;
                    $buttons = '<a href="'.$routeShow.'" type="button" name="view" class="btn btn-info btn-sm view"><i class="fas fa-search"> '.trans('backend.button.view').'</i></a>
                    <a href="'.$routeEdit.'" type="button" name="edit" class="btn btn-primary btn-sm edit"><i class="fas fa-pencil-alt"> '.trans('backend.button.edit').'</i></a> ';
                    $buttons .= '<button attr-delete="'.$routeDelete.'" type="button" name="delete" attr-id="'.$id.'" attr-slug="'.$slug.'" class="btn btn-danger btn-sm delete"><i class="fas fa-trash"> '.trans('backend.button.delete').'</i></button>';
                    return $buttons;
                })
                ->addColumn('active-icon', function($data){
                    return HelperService::getYesNoIconTable($data['active']);
                })
                ->addColumn('office-icon', function($data){
                    return HelperService::getYesNoIconTable($data['office']);
                })
                ->rawColumns(['action' => 'active-icon', 'office' => 'office-icon'])
                ->make(true);
            }
        }
        return DataTables::of(new Collection);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //Obtener los selects
        $offices = Office::all();
        $team = null;
        return view('backend.pages.team.form', ['offices' => $offices, 'team' => $team]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param TeamRequest $request
     * @return \Illuminate\Http\Response
     * @throws GeneralException
     */
    public function store(TeamRequest $request)
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $team = $this->teamService->createTeam($request);
                if ($team){
                    return redirect()->route('admin.auth.team.team.show', $team)->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('admin.auth.team.team.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $team = Team::where('id', $id)->first();
        if ($team) {
            return view('backend.pages.team.show', ['team' => $team]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
          //Obtener los selects
          $offices = Office::all();

          if ($offices->isEmpty()) {
             $offices = null;
         }

          $team = Team::where('id', $id)->first();
          if ($team) {
              return view('backend.pages.team.form', ['team' => $team, 'offices' => $offices]);
          } else {
              return redirect()->back()->withFlashDanger(__('auth.general_error'));
          }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param TeamRequest $request
     * @param int $id
     * @return \Illuminate\Http\Response
     * @throws ValidationException
     * @throws GeneralException
     */
    public function update(TeamRequest $request)
    {
          //
          $validator = null;
          try {
              if (isset($request->validator)) {
                  $validator = $request->validator;
              }
              if (!(isset($validator) && $validator->fails())) {

                  $team = $this->teamService->updateTeam($request);
                  if ($team){
                      return redirect()->route('admin.auth.team.team.show', $team)->withFlashSuccess(__('backend.alert.success'));
                  }
              }
          } catch (ValidationException $e) {
              throw new ValidationException($e->validator);
          } catch (Exception | Throwable $e) {
              Log::error($e);
              throw new GeneralException(__('auth.general_error'));
          }
        return redirect()->route('admin.auth.team.team.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $teamId
     * @return \Illuminate\Http\Response
     */
    public function delete($teamId)
    {
        $data = self::KO;
        $team = Team::where('id', $teamId)->first();

        if ($team && $team->id != 1) {
            if ($this->teamService->delete($team)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $teamId
     * @return \Illuminate\Http\Response
     */
    public function destroy($teamId)
    {
        $data = self::KO;
        $team = Team::where('id', $teamId)->first();

        if ($team && $team->id != 1) {
            if ($this->teamService->destroy($team)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

        /**
     * Remove the specified resource from storage.
     *
     * @param $teamId
     * @return \Illuminate\Http\Response
     */
    public function deleteAvatar($teamId){
        $data = self::KO;
        $team = Team::where('id', $teamId)->first();
        if ($team && $team->id != 1) {
            if ($this->teamService->deleteImage($team)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param MasterRequest $request
     * @return \Illuminate\Http\Response
     */
    public function saveUser(MasterRequest $request)
    {
        //
        $data = self::KO;

        if($request->ajax()){
            try {
                if (isset($request->validator)) {
                    $validator = $request->validator;
                }
                if (!(isset($validator) && $validator->fails())) {
                    $data = $this->masterService->saveEntity($request);
                }
            } catch (ValidationException $e) {
                $data = [self::KO => $e->getMessage()];
            } catch (Exception | Throwable $e) {
                Log::error($e);
                $data = [self::KO => __('auth.general_error')];
            }
        }

        return response()->json(json_encode($data, true));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param MasterRequest $request
     * @return \Illuminate\Http\Response
     */
    public function search(MasterRequest $request)
    {
        //
        $data = self::KO;

        if($request->ajax()){
            try {
                if (isset($request->validator)) {
                    $validator = $request->validator;
                }
                if (!(isset($validator) && $validator->fails())) {
                    $data = $this->masterService->getEntity($request);
                }
            } catch (ValidationException $e) {
                $data = [self::KO => $e->getMessage()];
            } catch (Exception | Throwable $e) {
                Log::error($e);
                $data = [self::KO => __('auth.general_error')];
            }
        }

        return response()->json(json_encode($data, true));
    }

}
