<?php

namespace App\Http\Controllers\Backend;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use Illuminate\Support\Collection;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;
use App\Services\UserService;
use App\Http\Requests\Backend\UserRequest;
use App\Models\User;
use App\Models\Company;
use App\Models\Country;
use App\Models\Team;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Throwable;

class UserController extends Controller implements StatusInterface
{

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('backend.pages.user.index');
    }

    /**
     * UserController constructor.
     *
     * @param  UserService  $userService
     *
     */
    public function __construct(UserService $userService)
    {
        $this->userService = $userService;
    }


    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function jsonList(Request $request)
    {
        if($request->ajax()){
            return $this->userService->getJsonList($request);
        }
        return DataTables::of(new Collection);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //Obtener los selects
        $companies = Company::all();
        $countries = Country::all();
        $teams = Team::all();
        $user = null;

        if ($companies->isEmpty()) {
            $companies = null;
        }
        if ($countries->isEmpty()) {
            $countries = null;
        }
        if ($teams->isEmpty()) {
            $teams = null;
        }
        return view('backend.pages.user.form', ['user' => $user, 'companies' => $companies, 'teams' => $teams, 'countries' => $countries]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param UserRequest $request
     * @return \Illuminate\Http\Response
     * @throws GeneralException
     */
    public function store(UserRequest $request)
    {
         //
         $validator = null;
         try {
             if (isset($request->validator)) {
                 $validator = $request->validator;
             }
             if (!(isset($validator) && $validator->fails())) {
                 $user = $this->userService->createUser($request);
                 if ($user){
                     return redirect()->route('admin.auth.user.user.show', $user)->withFlashSuccess(__('backend.alert.success'));
                 }
             }
         } catch (ValidationException $e) {
             return back()->withErrors($e->getMessage())->withInput();
         } catch (Exception | Throwable $e) {
             Log::error($e);
             throw new GeneralException(__('auth.general_error'));
         }
        return redirect()->route('admin.auth.user.user.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $user = User::where('id', $id)->first();
        if ($user) {
            return view('backend.pages.user.show', ['user' => $user]);
        } else {
            return redirect()->back()->withFlashDanger(__('auth.general_error'));
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
         //Obtener los selects
         $companies = Company::all();
         $countries = Country::all();
         $teams = Team::all();

         if ($companies->isEmpty()) {
            $companies = null;
        }
        if ($countries->isEmpty()) {
            $countries = null;
        }
        if ($teams->isEmpty()) {
            $teams = null;
        }
         $user = User::where('id', $id)->first();
         if ($user) {
             return view('backend.pages.user.form', ['user' => $user, 'companies' => $companies, 'teams' => $teams, 'countries' => $countries]);
         } else {
             return redirect()->back()->withFlashDanger(__('auth.general_error'));
         }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param UserRequest $request
     * @return \Illuminate\Http\Response
     * @throws ValidationException
     * @throws GeneralException
     */
    public function update(UserRequest $request)
    {
          //
          $validator = null;
          try {
              if (isset($request->validator)) {
                  $validator = $request->validator;
              }
              if (!(isset($validator) && $validator->fails())) {

                  $user = $this->userService->updateUser($request);
                  if ($user){
                      return redirect()->route('admin.auth.user.user.show', $user)->withFlashSuccess(__('backend.alert.success'));
                  }
              }
          } catch (ValidationException $e) {
              throw new ValidationException($e->validator);
          } catch (Exception | Throwable $e) {
              Log::error($e);
              throw new GeneralException(__('auth.general_error'));
          }
        return redirect()->route('admin.auth.user.user.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $userId
     * @return \Illuminate\Http\Response
     */
    public function destroy($userId)
    {
        $data = self::KO;
        $user = User::where('id', $userId)->first();
        if ($user) {
            if ($this->userService->destroy($user)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }

    /**
     * @param $userId
     * @return \Illuminate\Http\JsonResponse
     */
    public function delete($userId)
    {
        $data = self::KO;
        $user = User::where('id', $userId)->first();

        if ($user && $user->id != 1) {
            if ($this->userService->delete($user)) {
                $data = ['success' => self::OK];
            }
        }

        return response()->json(json_encode($data, true));
    }

        /**
     * Remove the specified resource from storage.
     *
     * @param $userId
     * @return \Illuminate\Http\Response
     */
    public function deleteAvatar($userId){
        $data = self::KO;
        $user = User::where('id', $userId)->first();
        if ($user) {
            if ($this->userService->deleteImage($user)) {
                $data = ['success' => self::OK];
            }
        }
        return response()->json(json_encode($data, true));
    }
}
