<?php

namespace App\Http\Controllers\Front;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Interfaces\StatusInterface;
use App\Http\Requests\Front\LeadRequest;
use App\Services\LeadService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Models\Lead;
use App\Models\Country;
use App\Models\Province;
use App\Models\Via;
use App\Models\Variant;
use App\Models\Periodicity;
use App\Models\Campaign;
use Exception;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Throwable;

class LeadController extends Controller implements StatusInterface
{

        /**
     * @var LeadService
     */
    protected $leadService;


    /**
     * LeadService constructor.
     *
     * @param  LeadService  $leadService
     *
     */
    public function __construct(LeadService $leadService)
    {
        $this->leadService = $leadService;
    }

    /**
     * Display a listing of the resource.
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        //Obtener la session
        $campaignId = $request->session()->get('campaign');
        $error = false;

        $user = Auth::User();
        if ($user->hasErrors()) {
            return redirect()->route('front.campaign.campaign.index')->withFlashDanger(__('front.campaign.message.no_delegations'));
        }
        $campaign = Campaign::where([['id', $campaignId], ['active', 1]])->first();
        if (!$campaignId || !$campaign) {
            return redirect()->route('front.campaign.campaign.index')->withFlashDanger(__('front.campaign.message.no_campaign'));
        } else {

            $lead = null;
            $client = null;
            $address = null;
            $leadLine = null;
            $variant = null;
            $periodicity = null;

            $countries = Country::all();
            $vias = Via::all();
            $provinces = Province::all();
            $variants = Variant::all();
            $periodicities = Periodicity::all();
            if ($countries->isEmpty()) {
                $countries = null;
            }
            if ($vias->isEmpty()) {
                $vias = null;
            }
            if ($provinces->isEmpty()) {
                $provinces = null;
            }
            if ($variants->isEmpty()) {
                $variants = null;
            }
            if ($periodicities->isEmpty()) {
                $periodicities = null;
            }
            return view('front.lead.index', ['lead' => $lead, 'client' => $client,
            'leadLine' => $leadLine, 'address' => $address,
            'countries' => $countries, 'vias' => $vias, 'provinces' => $provinces,
            'variants' => $variants, 'variant' => $variant,
            'periodicity' => $periodicity, 'periodicities' => $periodicities,
            'campaign' => $campaign]);
        }
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param LeadRequest $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws GeneralException
     */
    public function store(LeadRequest $request): \Illuminate\Http\RedirectResponse
    {
        //
        $validator = null;
        try {
            if (isset($request->validator)) {
                $validator = $request->validator;
            }
            if (!(isset($validator) && $validator->fails())) {
                $user = Auth::User();
                $errors = $user->hasErrors();
                if (isset($errors['status'])) {
                    throw new GeneralException($errors['message']);
                }
                $lead = $this->leadService->createLead($request);
                if ($lead){
                    return redirect()->route('front.lead.lead.thanks', ['lead' => $lead->id])->withFlashSuccess(__('backend.alert.success'));
                }
            }
        } catch (ValidationException $e) {
            return back()->withErrors($e->getMessage())->withInput();
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return redirect()->route('front.lead.lead.index')->withErrors(__('auth.general_error'))->withInput();
    }

    /**
     * @param Lead $lead
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function thanks(Lead $lead)
    {
        return view('front.lead.thanks', ['lead' => $lead]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id): \Illuminate\Http\Response
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id): \Illuminate\Http\Response
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id): \Illuminate\Http\Response
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id): \Illuminate\Http\Response
    {
        //
    }


    /**
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws GeneralException
     * @throws ValidationException
     */
    public function delega(Request $request)
    {
        try {
            $leads = null;

            $data = $request->all();
            if ($data) {
                $leads = $this->leadService->searchDelegate($request);
            }

        } catch (ValidationException $e) {
            throw new ValidationException($e->getMessage());
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return view('front.lead.delega', ['leads' => $leads]);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws GeneralException
     * @throws ValidationException
     */
    public function list(Request $request)
    {
        try {
            $leads = null;

            $data = $request->all();
            if ($data) {
                $leads = $this->leadService->searchRegister($request);
            }

        } catch (ValidationException $e) {
            throw new ValidationException($e->getMessage());
        } catch (Exception | Throwable $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }

        return view('front.lead.search', ['leads' => $leads]);
    }

}
