<?php

namespace App\Services;

use Illuminate\Contracts\Validation\Validator;
use Illuminate\Support\Facades\DB;
use App\Models\Announcement;
use Illuminate\Support\Facades\Log;
use App\Http\Requests\Backend\AnnouncementRequest;
use App\Exceptions\GeneralException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\ValidationException;
use App\Events\Announcement\AnnouncementCreated;
use App\Events\Announcement\AnnouncementUpdated;
use App\Events\Announcement\AnnouncementDeleted;
use App\Events\Announcement\AnnouncementDestroyed;
use Exception;
use Throwable;
use Carbon\Carbon;

/**
 * Class AnnouncementService.
 */
class AnnouncementService
{
    /**
     * @param Announcement $announcement
     *
     * @return bool
     */
    public function delete(Announcement $announcement): bool
    {
        try {
            DB::beginTransaction();
            $announcement->delete();
            $announcement->Save();
            //Logs
            Log::info('Borrat Anunci '.$announcement->id);
            event(new announcementDeleted($announcement));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

    /**
     * @param Announcement $announcement
     *
     * @return bool
     */
    public function destroy(Announcement $announcement): bool
    {
        try {
            DB::beginTransaction();
            $this->delete($announcement);
            $announcement->forceDelete();
            //Logs
            Log::info('Destruït Anunci '.$announcement->id);
            event(new announcementDestroyed($announcement));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

         /**
     * @param  AnnouncementRequest $request
     *
     * @return announcement
     * @throws GeneralException
     * @throws Throwable
     */
    public function updateAnnouncement(AnnouncementRequest $request): ?Announcement
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            //Buscar la announcement
            $announcement = null;
            $announcementId = isset($data['announcement-id']) ? $data['announcement-id'] : null;
            if ($announcementId) {
                $announcement = Announcement::where('id', $announcementId)->first();
                if ($announcement) {
                    $announcement = $this->manageAnnouncement($request, $announcement, $data);
                }
            }
            //Logs
            Log::info('Actualitzat Anuncu '.$announcement->id);
            event(new AnnouncementUpdated($announcement));
            DB::commit();
            return $announcement;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }



     /**
     * @param  AnnouncementRequest $request
     *
     * @return announcement
     * @throws GeneralException
     * @throws Throwable
     */
    public function createAnnouncement(AnnouncementRequest $request): ?Announcement
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            $announcement = new announcement();
            $announcement = $this->manageAnnouncement($request, $announcement, $data);
            //Logs
            Log::info('Creat Anunci '.$announcement->id);
            event(new AnnouncementCreated($announcement));
            DB::commit();
            return $announcement;
            //return false;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }

    /**
     * @param AnnouncementRequest $request
     * @param Announcement $announcement
     *
     * @param $data
     * @return Announcement
     * @throws ValidationException
     */
    private function manageAnnouncement($request, $announcement, $data) {
        $announcement->area = isset($data['area']) ? $data['area'] : null;
        $announcement->type = isset($data['type']) ? $data['type'] : null;
        $announcement->message = isset($data['message']) ? $data['message'] : null;
        $start = isset($data['start']) ? $data['start'] : null;
        $end = isset($data['end']) ? $data['end'] : null;
        $startDatetime = Carbon::createFromFormat('d/m/Y H:i', $start);
        $endDatetime = Carbon::createFromFormat('d/m/Y H:i', $end);
        if ($endDatetime->lte($startDatetime)) {
            throw ValidationException::withMessages(['end' => __('backend.announcement.form.ends_at')]);
        }
        $announcement->starts_at = $startDatetime;
        $announcement->ends_at = $endDatetime;
        $announcement->enabled = isset($data['enabled']) ? $data['enabled'] : false;
        $announcement->Save();
        return $announcement;
    }

    public function getForFront() {
        $now = Carbon::now('Europe/Madrid')->format('Y-m-d H:i');
        return Announcement::where([['enabled', 1], ['area', Announcement::AREA_FRONT],
        ['starts_at', '<=', $now], ['ends_at', '>=', $now]])->get();
    }

    public function getForBackend() {
        $now = Carbon::now('Europe/Madrid')->format('Y-m-d H:i');
        $valor = Announcement::where([['enabled', 1], ['area', Announcement::AREA_BACKEND],
        ['starts_at', '<=', $now], ['ends_at', '>=', $now]])->get();
        return $valor;
    }
}
