<?php

namespace App\Services;

use Illuminate\Contracts\Validation\Validator;
use Illuminate\Support\Facades\DB;
use App\Events\Company\CompanyUpdated;
use App\Events\Company\CompanyCreated;
use App\Events\Company\CompanyDeleted;
use App\Events\Company\CompanyDestroyed;
use App\Models\CompanyLink;
use App\Models\Company;
use App\Models\Master;
use App\Models\User;
use App\Models\Province;
use App\Models\Address;
use Illuminate\Support\Facades\Log;
use App\Http\Requests\Backend\CompanyRequest;
use App\Exceptions\GeneralException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\ValidationException;
use Image;
use Exception;
use Throwable;
use App\Services\HelperService;

/**
 * Class CompanyService.
 */
class CompanyService
{
    /**
     * @param Company $company
     *
     * @return bool
     */
    public function delete(Company $company): bool
    {
        try {
            DB::beginTransaction();
            //Borrar los links
            CompanyLink::where('company_id', $company->id)->delete();
            //Borrar usuarios
            $users =  $company->users;
            foreach($users as $users) {
                $users->company_id = null;
                $users->Save();
            }
            //Quitar la referencia con el usuario
            User::where('company_id', '=', $company->id)->update(['company_id' => null]);
            //Borrar las relaciones con usuarios
            $company->masters()->delete();
            $company->masters()->detach();

            //Quitar Address
            $company->address()->delete();
            $company->address()->detach();

            //Borrar las relaciones con campanas
            $campaigns =  $company->campaigns;
            foreach($campaigns as $campaign) {
                $campaign->company_id = null;
                $campaign->Save();
            }
            //Borrar la empresa
            $delegations =  $company->delegations;
            foreach($delegations as $delegation) {
                $offices = $delegation->offices;
                foreach($offices as $office) {
                    $teams = $office->teams;
                    foreach($teams as $team) {
                        $team->office_id = null;
                        $team->Save();
                    }
                    $office->delegation_id = null;
                    $office->Save();
                }
            $delegation->company_id = null;
            $delegation->Save();
            }

            $company->delete();
            $company->Save();
            //Logs
            Log::info('Borrada la empresa '.$company->id);
            event(new CompanyDeleted($company));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

        /**
     * @param Company $company
     *
     * @return bool
     */
    public function deleteImage(Company $company): bool
    {
        try {
            DB::beginTransaction();
            $company->image_location=NULL;
            $company->Save();
            //Logs
            Log::info('Borrada la imagen '.$company->id);
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

    /**
     * @param Company $company
     *
     * @return bool
     */
    public function destroy(Company $company): bool
    {
        try {
            DB::beginTransaction();
            $this->delete($company);
            CompanyLink::where('company_id', $company->id)->forceDelete();

            $company->forceDelete();
            //Logs
            Log::info('Destruida la empresa '.$company->id);
            event(new CompanyDestroyed($company));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

         /**
     * @param  CompanyRequest $request
     *
     * @return Company
     * @throws GeneralException
     * @throws Throwable
     */
    public function updateCompany(CompanyRequest $request): ?Company
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            //Buscar la empresa
            $company = null;
            $companyId = isset($data['company-id']) ? $data['company-id'] : null;
            if ($companyId) {
                $company = Company::where('id', $companyId)->first();
                if ($company) {
                    $company = $this->manageCompany($request, $company, $data);
                }
            }
            if (!$company) {
                throw new GeneralException(__('auth.general_error'));
            }
            //Logs
            Log::info('Actualizada la empresa '.$company->id);
            event(new CompanyUpdated($company));
            DB::commit();
            return $company;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }



     /**
     * @param  CompanyRequest $request
     *
     * @return Company
     * @throws GeneralException
     * @throws Throwable
     */
    public function createCompany(CompanyRequest $request): ?Company
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            $company = new Company();
            $company = $this->manageCompany($request, $company, $data);
            //Logs
            Log::info('Creada la empresa '.$company->id);
            event(new CompanyCreated($company));
            DB::commit();
            return $company;
            //return false;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }

    /**
     * @param $request
     * @param $company
     * @param $data
     * @return mixed
     * @throws ValidationException
     */
    private function manageCompany($request, $company, $data) {

        $company->code = isset($data['code']) ? $data['code'] : null;
        $company->name = isset($data['name']) ? $data['name'] : null;
        $company->slug = isset($data['name']) ? str_slug($data['name'], "-") : null;
        $company->web = isset($data['web']) ? $data['web'] : null;
        $company->email = isset($data['email']) ? $data['email'] : null;
        $company->phone = isset($data['phone']) ? $data['phone'] : null;
        $company->mobile = isset($data['mobile']) ? $data['mobile'] : null;
        $company->register = isset($data['register']) ? $data['register'] : null;
        $company->conditions = isset($data['conditions']) ? $data['conditions'] : null;
        $nif = isset($data['nif']) ? $data['nif'] : null;
        if (!HelperService::validateNif($nif)) {
            throw new GeneralException(__('auth.general_error'));
        }
        $company->nif = $nif;
        $iban = isset($data['iban']) ? $data['iban'] : null;
        $account = isset($data['account']) ? $data['account'] : null;
        $cuenta = strtoupper($iban.$account);
        if (!HelperService::validateAccount($cuenta)) {
            throw new GeneralException(__('auth.general_error'));
        }
        $company->account = $cuenta;

        $company->active = isset($data['active']) ? $data['active'] : false;

        $company->Save();
        $companyId = $company->id;
        $company = HelperService::updateAddress($company, $data);

        //Insertar avatar
        $filename = null;
        if ($request->hasFile('avatar')) {
            $file = $request->file('avatar');
            $filename = HelperService::uploadFile($file, 'cavatar', $companyId);
            $company->image_location = $filename;
        }
        $company->Save();
        return $company;
    }
}
