<?php

namespace App\Services;

use Illuminate\Contracts\Validation\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use App\Events\Delegation\DelegationUpdated;
use App\Events\Delegation\DelegationCreated;
use App\Events\Delegation\DelegationDeleted;
use App\Events\Delegation\DelegationDestroyed;
use App\Models\User;
use App\Models\Delegation;
use Illuminate\Support\Facades\Log;
use App\Http\Requests\Backend\DelegationRequest;
use App\Exceptions\GeneralException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\ValidationException;
use Image;
use Exception;
use Throwable;
use App\Services\HelperService;
use Illuminate\Support\Facades\Session;

/**
 * Class DelegationService.
 */
class DelegationService
{
/**
     * @param Delegation $delegation
     *
     * @return bool
     */
    public function delete(Delegation $delegation): bool
    {
        try {
            DB::beginTransaction();

            $this->updateRelationDelegation($delegation);
            //Logs
            Log::info('Borrada delegacion '.$delegation->id);
            event(new DelegationDeleted($delegation));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

        /**
     * @param Delegation $delegation
     *
     * @return bool
     */
    public function deleteImage(Delegation $delegation): bool
    {
        try {
            DB::beginTransaction();
            $delegation->image_location=NULL;
            $delegation->Save();
            //Logs
            Log::info('Borrada la imagen '.$delegation->id);
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

    /**
     * @param Delegation $delegation
     *
     * @return bool
     */
    public function destroy(Delegation $delegation): bool
    {
        try {
            DB::beginTransaction();
            $this->delete($delegation);
            $delegation->forceDelete();
            //Logs
            Log::info('Destruida delegacion '.$delegation->id);
            event(new DelegationDestroyed($delegation));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

             /**
     * @param  DelegationRequest $request
     *
     * @return Delegation
     * @throws GeneralException
     * @throws Throwable
     */
    public function updateDelegation(DelegationRequest $request): ?Delegation
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            //Buscar el equipo
            $delegation = null;
            $delegationId = isset($data['delegation-id']) ? $data['delegation-id'] : null;
            if ($delegationId) {
                $delegation = Delegation::where('id', $delegationId)->first();
                if ($delegation) {
                    $delegation = $this->manageDelegation($request, $delegation, $data);
                    $delegation = $this->updateRelationDelegation($delegation, true);
                }
            }
            if (!$delegation) {
                throw new GeneralException(__('auth.general_error'));
            }
            //Logs
            Log::info('Actualizada delegacion '.$delegation->id);
            event(new DelegationUpdated($delegation));
            DB::commit();
            return $delegation;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }


     /**
     * @param  DelegationRequest $request
     *
     * @return Delegation
     * @throws GeneralException
     * @throws Throwable
     */
    public function createDelegation(DelegationRequest $request): ?Delegation
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            $delegation = new Delegation();
            $delegation = $this->manageDelegation($request, $delegation, $data);
            //Logs
            Log::info('Creada la delegacion '.$delegation->id);
            event(new DelegationCreated($delegation));
            DB::commit();
            return $delegation;
            //return false;
        } catch (ValidationException $e) {
                throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }



    /**
     * @param DelegationRequest $request
     * @param Delegation $delegation
     *
     * @param $data
     * @return Delegation
     * @throws ValidationException
     */
    private function manageDelegation($request, $delegation, $data) {

        $delegation->name = isset($data['name']) ? $data['name'] : null;
        $delegation->slug = isset($data['name']) ? str_slug($data['name'], "-") : null;
        $delegation->company_id = isset($data['company']) ? $data['company'] : null;
        $delegation->active = isset($data['active']) ? $data['active'] : false;

        $delegation->Save();
        $delegationId = $delegation->id;
        //Insertar avatar
        if ($request->hasFile('avatar')) {
            $file = $request->file('avatar');
            $filename = HelperService::uploadFile($file, 'davatar', $delegationId);
            $delegation->image_location = $filename;
        }
        $delegation->Save();
        return $delegation;
    }

    /**
     * @param $delegation
     * @param false $update
     * @return mixed
     */
    private function updateRelationDelegation($delegation, $update = false){
        //Borrar usuarios
        $users =  $delegation->users;
        foreach($users as $users) {
            if ($update) {
                $users->delegation_id = $delegation->id;
                User::where('delegation_id', '=', $delegation->id)->update(['delegation_id' => $delegation->id]);
                $users->Save();
            } else {
                $users->delegation_id = null;
                User::where('delegation_id', '=', $delegation->id)->update(['delegation_id' => null]);
                $users->Save();
            }
        }

        $masters = $delegation->masters;
        if ($update) {
            if (count($delegation->getChanges())) {
                $changes = $delegation->getChanges();
                if (isset($changes['company_id'])) {
                    Session::flash('flash_message', trans('backend.delegation.message.update'));
                    foreach($masters as $master) {
                    //Borrar las relaciones con usuarios
                        $delegation->masters()->delete();
                        $delegation->masters()->detach();
                    }
                }
            }
        } else {
            foreach($masters as $master) {
            //Borrar las relaciones con usuarios
            $delegation->masters()->delete();
            $delegation->masters()->detach();
            }
            //Borrar la delegacion y el resto
            $offices = $delegation->offices;
            foreach($offices as $office) {
                $teams = $office->teams;
                foreach($teams as $team) {
                    $team->office_id = null;
                    $team->Save();
                }
                $office->delegation_id = null;
                $office->Save();
            }
            $delegation->delete();
            $delegation->Save();
        }
        return $delegation;
    }
}
