<?php

namespace App\Services;

use Illuminate\Contracts\Validation\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use App\Events\Office\OfficeUpdated;
use App\Events\Office\OfficeCreated;
use App\Events\Office\OfficeDeleted;
use App\Events\Office\OfficeDestroyed;
use App\Models\User;
use App\Models\Office;
use Illuminate\Support\Facades\Log;
use App\Http\Requests\Backend\OfficeRequest;
use App\Exceptions\GeneralException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\ValidationException;
use Image;
use Exception;
use Throwable;
use App\Services\HelperService;
use Illuminate\Support\Facades\Session;

/**
 * Class OfficeService.
 */
class OfficeService
{
/**
     * @param Office $office
     *
     * @return bool
     */
    public function delete(Office $office): bool
    {
        try {
            DB::beginTransaction();

            $this->updateRelationOffice($office);
            //Logs
            Log::info('Borrada oficina '.$office->id);
            event(new OfficeDeleted($office));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

        /**
     * @param Office $office
     *
     * @return bool
     */
    public function deleteImage(Office $office): bool
    {
        try {
            DB::beginTransaction();
            $office->image_location=NULL;
            $office->Save();
            //Logs
            Log::info('Borrada la imagen '.$office->id);
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

    /**
     * @param Office $office
     *
     * @return bool
     */
    public function destroy(Office $office): bool
    {
        try {
            DB::beginTransaction();
            $this->delete($office);
            $office->forceDelete();
            //Logs
            Log::info('Destruida oficina '.$office->id);
            event(new OfficeDestroyed($office));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

             /**
     * @param  OfficeRequest $request
     *
     * @return Office
     * @throws GeneralException
     * @throws Throwable
     */
    public function updateOffice(OfficeRequest $request): ?Office
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            //Buscar el equipo
            $office = null;
            $officeId = isset($data['office-id']) ? $data['office-id'] : null;
            if ($officeId) {
                $office = Office::where('id', $officeId)->first();
                if ($office) {
                    $office = $this->manageOffice($request, $office, $data);
                    $office = $this->updateRelationOffice($office, true);
                }
            }
            if (!$office) {
                throw new GeneralException(__('auth.general_error'));
            }
            //Logs
            Log::info('Actualizada oficina '.$office->id);
            event(new OfficeUpdated($office));
            DB::commit();
            return $office;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }


     /**
     * @param  OfficeRequest $request
     *
     * @return Office
     * @throws GeneralException
     * @throws Throwable
     */
    public function createOffice(OfficeRequest $request): ?Office
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            $office = new Office();
            $office = $this->manageOffice($request, $office, $data);
            //Logs
            Log::info('Creada la oficina '.$office->id);
            event(new OfficeCreated($office));
            DB::commit();
            return $office;
            //return false;
        } catch (ValidationException $e) {
                throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }



    /**
     * @param OfficeRequest $request
     * @param Office $office
     *
     * @param $data
     * @return Office
     * @throws ValidationException
     */
    private function manageOffice($request, $office, $data) {

        $office->name = isset($data['name']) ? $data['name'] : null;
        $office->slug = isset($data['name']) ? str_slug($data['name'], "-") : null;
        $office->delegation_id = isset($data['delegation']) ? $data['delegation'] : null;
        $office->active = isset($data['active']) ? $data['active'] : false;

        $office->Save();
        $officeId = $office->id;
        //Insertar avatar
        if ($request->hasFile('avatar')) {
            $file = $request->file('avatar');
            $filename = HelperService::uploadFile($file, 'oavatar', $officeId);
            $office->image_location = $filename;
        }
        $office->Save();
        return $office;
    }

    /**
     * @param $office
     * @param false $update
     * @return mixed
     */
    private function updateRelationOffice($office, $update = false){
        //Borrar usuarios
        $users =  $office->users;
        foreach($users as $users) {
            if ($update) {
                $users->office_id = $office->id;
                User::where('office_id', '=', $office->id)->update(['office_id' => $office->id]);
                $users->Save();
            } else {
                $users->office_id = null;
                User::where('office_id', '=', $office->id)->update(['office_id' => null]);
                $users->Save();
            }
        }

        $masters = $office->masters;
        if ($update) {
            if (count($office->getChanges())) {
                $changes = $office->getChanges();
                if (isset($changes['delegation_id'])) {
                    Session::flash('flash_message', trans('backend.office.message.update'));
                    foreach($masters as $master) {
                    //Borrar las relaciones con usuarios
                        $office->masters()->delete();
                        $office->masters()->detach();
                    }
                }
            }
        } else {
            foreach($masters as $master) {
            //Borrar las relaciones con usuarios
            $office->masters()->delete();
            $office->masters()->detach();
            }
            //Borrar la oficina y el resto
            $teams = $office->teams;
            foreach($teams as $team) {
                $team->office_id = null;
                $team->Save();
            }
            $office->delete();
            $office->Save();
        }
        return $office;
    }
}
