<?php

namespace App\Services;

use Illuminate\Contracts\Validation\Validator;
use Illuminate\Support\Facades\DB;
use App\Events\Register\RegisterCreated;
use App\Events\Register\RegisterStateUpdated;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;
use App\Http\Requests\Front\SearchRequest;
use App\Http\Requests\Front\RegisterRequest;
use App\Http\Requests\Backend\RegisterSearchRequest;
use App\Exceptions\GeneralException;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use Carbon\CarbonPeriod;
use App\Models\Register;
use App\Models\RegisterState;
use App\Models\RegisterStateReviewer;
use App\Models\RegisterIP;
use Torann\GeoIP;
use DataTables;
use Throwable;
use App\Services\HelperService;
use Browser;
use App\Http\Controllers\Interfaces\StatusInterface;

/**
 * Class RegisterService.
 */
class RegisterService implements StatusInterface
{
     /**
     * @param  RegisterRequest $request
     *
     * @throws GeneralException
     * @throws Throwable
     */
    public function createRegister(RegisterRequest $request): ?array
    {
        try {
            DB::beginTransaction();

            $formData = $request->all();

            $tipo = isset($formData['type']) ? strtoupper($formData['type']) :null;
            $comments = isset($formData['comments']) ? $formData['comments']: '';
            $commentsClean = strip_tags($comments);
            $commentsClean = preg_replace("/\xEF\xBB\xBF/", "", $commentsClean);
            if (strlen($commentsClean) == 0) {
                $comments = '';
            }
            $holidays = isset($formData['holidays']) ? strtoupper($formData['holidays']) :null;
            $manual = isset($formData['manual']) ? strtoupper($formData['manual']) :null;
            $user = Auth::User();
            if (!$user || !$tipo) {
                return [self::ERROR => __('front.hour.error.user')];
            }
            //Obtener la empresa
            $company = $user->myCompany;
            if (!$company) {
                //Error
                return [self::ERROR => __('front.hour.error.company')];
            }

            //Si llega holidays entonces ingresar el registro
            if ($tipo == Register::REGISTER_TYPE_HOLIDAYS) {
                if (!$holidays)  {
                    return [self::ERROR => __('front.hour.error.holidays')];
                }
                $holidaysArray = explode("-", $holidays);
                if (!$holidaysArray || count($holidaysArray) != 2) {
                    return [self::ERROR => __('front.hour.error.holidays')];
                }
                $holidayStartString =trim($holidaysArray[0]);
                $holidayEndString =trim($holidaysArray[1]);
                $holidayStart = Carbon::createFromFormat('d/m/Y', $holidayStartString);
                $holidayEnd = Carbon::createFromFormat('d/m/Y', $holidayEndString);
                //El dia de inicio es mayor que el dia de fin
                if ($holidayStart->gt($holidayEnd)) {
                    return [self::ERROR => __('front.hour.error.holidays')];
                }
                //Buscar si el usuario tiene fechas ya cogidas
                $resgisterHolidays = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                ->where([['register_states.user_id', $user->id],['register_states.company_id', $company->id],
                ['hour', '>=', $holidayStart->format('Y-m-d')], ['hour', '<=', $holidayEnd->format('Y-m-d')]])->first();
                if ($resgisterHolidays) {
                    return [self::ERROR => __('front.hour.error.holidays_duplicated', ['date' => $holidays])];
                }
                $period = CarbonPeriod::create($holidayStart, $holidayEnd);
                $registerState = new RegisterState();
                $registerState->state_type = Register::REGISTER_TYPE_CLOSED;
                $registerState->user_id = $user->id;
                $registerState->company_id = $company->id;
                $registerState->Save();
                $registerStateId = $registerState->id;
                $registerDateTime = $holidayStart->format('Y-m-d H:i');
                foreach ($period as $dateHoliday) {
                    //Crear los registros
                    $register = new Register();
                    $register->type = $tipo;
                    $register->date = $dateHoliday->format('Y-m-d');
                    $register->hour = $dateHoliday->format('Y-m-d');
                    $register->register_state_id = $registerStateId;
                    $register->comments = $comments;
                    $register->save();
                }
            } else {

                if ($manual) {
                    $registerDateTime = Carbon::createFromFormat('d/m/Y H:i', $manual);
                    $registerDate = $registerDateTime->format('Y-m-d');
                    $registerDateTime = $registerDateTime->toDateTimeString();
                } else {
                    $registerDate = Carbon::now('Europe/Madrid')->format('Y-m-d');
                    $registerDateTime = Carbon::now('Europe/Madrid')->toDateTimeString();
                }

                if ($tipo == Register::REGISTER_TYPE_START) {
                    $registerDateTimeEnd = Carbon::parse($registerDateTime)->addHour()->format('Y-m-d H:i');
                    //El registro de start no puede tener otro siguiente en las siguiente hora
                    $resgisterLater =  Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                    ->where([['register_states.user_id', $user->id],
                    ['register_states.company_id', $company->id],
                    ['hour', '>', $registerDateTime], ['hour', '<', $registerDateTimeEnd], ['type', '<>', Register::REGISTER_TYPE_HOLIDAYS]])->first();
                    if ($resgisterLater) {
                        return [self::ERROR => __('front.hour.error.no_consistent')];
                    }
                } else {
                    //Revisar que entre los registros actuales y el que nuevo no hay registros
                    $registerRegisterState = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                    ->where([['user_id', $user->id], ['company_id', $company->id]])
                    ->Where('register_states.state_type', Register::REGISTER_TYPE_START)
                    ->orWhere('register_states.state_type', Register::REGISTER_TYPE_STOP)
                    ->orWhere('register_states.state_type', Register::REGISTER_TYPE_RETURN)
                    ->get()->last();
                    if ($registerRegisterState) {
                        $registerDateTimeInserted = $registerRegisterState->hour;
                        $start = '';
                        $finish = '';
                        if (Carbon::parse($registerDateTime)->gte(Carbon::parse($registerDateTimeInserted))){
                            $start = $registerDateTimeInserted;
                            $finish = $registerDateTime;
                        } else {
                            $start = $registerDateTime;
                            $finish = $registerDateTimeInserted;
                        }
                        $resgisterBetween = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                        ->where([['register_states.user_id', $user->id],
                        ['register_states.company_id', $company->id],
                        ['hour', '>', $start], ['hour', '<', $finish]])->first();
                        if ($resgisterBetween) {
                            return [self::ERROR => __('front.hour.error.no_consistent')];
                        }
                    }
                }

                $findState = '';
                if ($tipo == Register::REGISTER_TYPE_START) {
                    //Ver si el chivato del registro del usuario esta como FINISH
                    $findState = [Register::REGISTER_TYPE_CLOSED];

                    if (!$manual) {
                        //El registro ha de ser posterior a cualquier otro para ese usuario y empresa
                        $resgisterLater = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                        ->where([['register_states.user_id', $user->id],
                        ['register_states.company_id', $company->id],['closed', 0],
                        ['hour', '>', $registerDateTime], ['type', '<>', Register::REGISTER_TYPE_HOLIDAYS]])->first();
                        if ($resgisterLater) {
                            return [self::ERROR => __('front.hour.error.no_consistent')];
                        }
                    }

                } elseif ($tipo == Register::REGISTER_TYPE_STOP) {
                    //Ver si el chivato del registro del usuario esta como START
                    $findState = [Register::REGISTER_TYPE_START];
                } elseif ($tipo == Register::REGISTER_TYPE_RETURN) {
                    //Ver si el chivato del registro del usuario esta como STOP
                    $findState = [Register::REGISTER_TYPE_STOP];
                } elseif ($tipo == Register::REGISTER_TYPE_FINISH) {
                    //Ver si el chivato del registro del usuario esta como RETURN O START
                    $findState = [Register::REGISTER_TYPE_START, Register::REGISTER_TYPE_RETURN];
                }
                //Registro no duplicado
                $registerRegisterStatesDuplicated = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                ->where([['register_states.user_id', $user->id], ['register_states.company_id', $company->id],['closed', 0],
                ['register_states.state_type', '=', $tipo]])
                ->first();
                if ($registerRegisterStatesDuplicated) {
                    return [self::ERROR => __('front.hour.error.duplicated', ['register' => $tipo])];
                }

                //Registro anterior no existente
                $registerRegisterStates = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                ->where([['register_states.user_id', $user->id], ['register_states.company_id', $company->id], ['register_states.closed', 0]])
                ->whereIn('register_states.state_type', $findState)
                ->first();
                if (!$registerRegisterStates) {
                    //Puede ser el primero
                    $registerRegisterStatesFirst = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                    ->where([['register_states.user_id', $user->id], ['register_states.company_id', $company->id],['register_states.closed', 0]])
                    ->first();
                    if ($registerRegisterStatesFirst) {
                        return [self::ERROR => __('front.hour.error.no_register', ['register' => implode(",", $findState)])];
                    }
                }

                //Verificar los tipos
                if ($tipo == Register::REGISTER_TYPE_START) {
                    $registerStateJoin = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                    ->where([['register_states.user_id', $user->id], ['register_states.company_id', $company->id], ['register_states.closed', 0]])
                    ->whereIn('register_states.state_type', [Register::REGISTER_TYPE_RETURN, Register::REGISTER_TYPE_STOP])
                    ->select('register_state_id')->first();
                    if ($registerStateJoin) {
                        return [self::ERROR => __('front.hour.error.no_register', ['register' => implode(",", $findState)])];
                    }
                    //Crear un id de registro de estado
                    $registerState = new RegisterState();
                    $registerState->state_type = $tipo;
                    $registerState->user_id = $user->id;
                    $registerState->company_id = $company->id;
                    $registerState->Save();
                    $registerStateId = $registerState->id;
                } else {
                    //Buscar el id del chivato del registro del usuario
                    $registerStateJoin = Register::join('register_states', 'register_states.id', '=', 'register_state_id')
                    ->where([['register_states.user_id', $user->id], ['register_states.company_id', $company->id],['register_states.closed', 0],
                    ['register_states.state_type', '<>', Register::REGISTER_TYPE_CLOSED]])->
                    select('register_state_id')->first();
                    if ($registerStateJoin) {
                        $registerStateId = $registerStateJoin->register_state_id;
                    } else {
                        return [self::ERROR => __('front.hour.error.no_register', ['register' => implode(",", $findState)])];
                    }
                    $registerState = RegisterState::where('id', $registerStateId)->first();
                    if ($tipo == Register::REGISTER_TYPE_FINISH) {
                        $registerState->state_type = Register::REGISTER_TYPE_CLOSED;
                    } else {
                        $registerState->state_type = $tipo;
                    }
                    $registerState->Save();
                }

                $register = new Register();
                $register->type = $tipo;
                $register->date = $registerDate;
                $register->hour = $registerDateTime;
                $register->register_state_id = $registerStateId;
                $register->comments = $comments;
                $register->save();
            }
            //Logs
            Log::info('Creado el registro '.$register->id);
            event(new RegisterCreated($register));
            DB::commit();
            //Register IP
            $ipAddress = request()->getClientIp();
            $userAgent = $request->header('User-Agent');
            if (isset($register)) {
                $this->saveRegisterIp($ipAddress, $userAgent, $register);
            }
            return [self::SUCCESS => __('front.hour.message.success', ['register' => $tipo, 'date' => $registerDateTime])];
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }

    /**
     * @param $ipAddress
     * @param $userAgent
     * @param $register
     */
    private function saveRegisterIp($ipAddress, $userAgent, $register){
         //GeoIP
         $geoip = geoip($ipAddress);
         if ($geoip) {
             $registerIp = new RegisterIP();
             $registerIp->register_id = $register->id;
             $registerIp->ip = $ipAddress;
             $registerIp->iso_code = $geoip['iso_code'];
             $registerIp->country = $geoip['country'];
             $registerIp->city = $geoip['city'];
             $registerIp->state = $geoip['state'];
             $registerIp->state_name = $geoip['state_name'];
             $registerIp->postal_code = $geoip['postal_code'];
             $registerIp->lat = $geoip['lat'];
             $registerIp->long = $geoip['lon'];
             $registerIp->timezone = $geoip['timezone'];
             $registerIp->continent = $geoip['continent'];
             $registerIp->currency = $geoip['currency'];
             $registerIp->user_agent = $userAgent;
             $registerIp->browser = json_encode(['user_agent'=>Browser::userAgent(),
             'is_mobile'=>Browser::isMobile(),
             'is_desktop'=>Browser::isDesktop(),
             'is_tablet'=>Browser::isTablet(),
             'browser_name'=>Browser::browserName(),
             'platform_name'=>Browser::platformName(),
             'platform_family'=>Browser::platformFamily(),
             'platform_version'=>Browser::platformVersion(),
             'device_family'=>Browser::deviceFamily(),
             'device_model'=>Browser::deviceModel()
             ]);
             $registerIp->save();
         }
    }

        /**
     * @param RegisterSearchRequest $request
     * @return mixed
     */
    public function getJsonList(RegisterSearchRequest $request) {
        $data = $request->all();
        $type = isset($data['radio-select']) ? strtoupper($data['radio-select']) : null;
        $dataCollection = [];
        $registerStates = null;
        if ($type == RegisterState::REGISTER_STATE_ALL) {
            $registerStates = RegisterState::where([['state', RegisterState::REGISTER_STATE_PDTE, 'closed' => 0]])->distinct()->get();
        }
        if ($registerStates && count($registerStates)>0) {
            foreach($registerStates as $registerState) {
                //Obtener los datos de los registros
                $user = ($registerState->myUser != null) ? $registerState->myUser : null;
                $username = '';
                if ($user) {
                    $username = $user->username;
                }
                $company = ($registerState->myCompany != null) ? $registerState->myCompany : null;
                if ($company) {
                    $company = $company->name;
                }
                $closed = false;
                if ($registerState->state_type == Register::REGISTER_TYPE_CLOSED) {
                    $closed = true;
                }
                $type = $registerState->state_type;
                $typeName = $this->getTypeName($type);
                $state = $registerState->state;
                $stateName = $this->getStateName($state);
                $date = Carbon::parse($registerState->created_at)->format('d/m/Y H:i:s');
                $comments = '';

                $registers = $registerState->registers;
                foreach($registers as $register) {
                    $comments .= $register->comments;
                }
                $comments = strip_tags($comments);
                if (strlen($comments) > 15) {
                    $comments = substr($comments, 0, 15);
                }
                $dataCollection[]  = [
                    'id'         => $registerState->id,
                    'user'       => $username,
                    'company'    => $company,
                    'date'       => $date,
                    'type'       => $typeName,
                    'comments'   => $comments,
                    'status'     => $stateName,
                    'diff'       => Carbon::parse($registerState->created_at)->diffForHumans(),
                    'closed'     => $closed
                ];
            }
            return [self::SUCCESS => self::SUCCESS, 'data' => $dataCollection];
        }
        return [self::SUCCESS => self::SUCCESS];
    }

    /**
     * @param $type
     * @return array|\Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Translation\Translator|string|null
     */
    private function getTypeName($type)
    {
        $typeName = '';
        switch ($type) {
            case 'START':
                $typeName = __('backend.hour.form.start');
                break;
            case 'STOP':
                $typeName = __('backend.hour.form.stop');
                break;
            case 'RETURN':
                $typeName = __('backend.hour.form.return');
                break;
            case 'FINISH':
                $typeName = __('backend.hour.form.finish');
                break;
            case 'HOLIDAYS':
                $typeName = __('backend.hour.form.holidays');
                break;
            case 'CLOSED':
                $typeName = __('backend.hour.form.pdte');
                break;
            default:
                $typeName = '';
                break;
        }
        return $typeName;
    }

    /**
     * @param string $stateName
     * @return array|\Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Translation\Translator|string|null
     */
    private function getStateName(string $stateName)
    {
        switch ($stateName) {
            case 'PDTE':
                $stateName = __('backend.hour.form.pdte');
                break;
            case 'OK':
                $stateName = __('backend.hour.form.ok');
                break;
            case 'KO':
                $stateName = __('backend.hour.form.ko');
                break;
            default:
                $stateName = '';
                break;
        }
        return $stateName;
    }

        /**
     * @param Request $request
     * @return mixed
     */
    public function stateRegister(Request $request) {
        try {
            DB::beginTransaction();

            $formData = $request->all();

            $tipo = isset($formData['type']) ? strtoupper($formData['type']) :null;
            $ids = isset($formData['ids']) ? $formData['ids'] :null;

            if (!$tipo) {
                return [self::ERROR => __('auth.general_error')];
            }
            $state = '';
            switch($tipo) {
                case RegisterState::REGISTER_STATE_OK:
                    $state = RegisterState::REGISTER_STATE_OK;
                break;
                case RegisterState::REGISTER_STATE_KO:
                    $state = RegisterState::REGISTER_STATE_KO;
                break;
                default:
                    return [self::ERROR => __('auth.general_error')];
                break;
            }
            foreach ($ids as $id) {
                $registerState = RegisterState::where('id', $id)->first();
                $registerState->update( ['closed' => 1]);
                $registerState->update( ['state' => $state]);
                $registerState->Save();

                //Actualizar la info del reviewer
                $registerStateReviewer = new RegisterStateReviewer();
                $registerStateReviewer->register_states_id = $registerState->id;
                $registerStateReviewer->reviewer_id = Auth::User()->id;
                $registerStateReviewer->data = json_encode($formData);
                $registerStateReviewer->save();
                //Logs
                Log::info('Actualizando el RegisterState '.$registerState->id);
                event(new RegisterStateUpdated($registerState));
            }
            DB::commit();
            return [self::SUCCESS => __('backend.hour.form.ok')];
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }

    /**
     * @param Request $request
     * @return array|null
     * @throws GeneralException
     */
    public function getLocalization(Request $request): ?array
    {
        try {
            $data = $request->all();
            $id = isset($data['id']) ? $data['id'] : null;

            if ($id) {
                $registerIP = RegisterIP::where('register_id', $id)->first();
                $browser = json_decode($registerIP->browser, true);
                $device = '';
                if (isset($browser['is_mobile']) && $browser['is_mobile']) {
                    $device = 'Mobile';
                } elseif (isset($browser['is_desktop']) && $browser['is_desktop']) {
                    $device = 'Ordenador';
                } elseif (isset($browser['is_tablet']) && $browser['is_tablet']) {
                    $device = 'Tablet';
                }
                $navegador = '';
                if (isset($browser['browser_name'])) {
                    $navegador = $browser['browser_name'];
                }
                $platform = '';
                if (isset($browser['platform_name'])) {
                    $platform = $browser['platform_name'];
                }
                $data = [
                    'id'         => $registerIP->id,
                    'register'   => Carbon::parse($registerIP->created_at)->format('d/m/Y H:i:s'),
                    'ip'         => $registerIP->ip,
                    'country'    => $registerIP->country,
                    'city'       => $registerIP->city,
                    'lat'        => $registerIP->lat,
                    'long'       => $registerIP->long,
                    'state'      => $registerIP->state,
                    'cp'         => $registerIP->postal_code,
                    'device'     => $device,
                    'browser'    => $navegador,
                    'platform'   => $platform,
                    'diff'       => Carbon::parse($registerIP->created_at)->diffForHumans(),
                    'timezone'   => $registerIP->timezone,
                    'currency'   => $registerIP->currency,
                ];
                return [self::SUCCESS => self::SUCCESS, 'data' => $data];
            }
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
        }
        return null;
    }


    /**
     * @param SearchRequest $request
     * @return array|null
     */
    public function searchRegister(SearchRequest $request): ?array
    {
        $formData = $request->all();

        $dataCollection = [];

        $start = isset($formData['start']) ? $formData['start'] : null;
        $finish = isset($formData['finish']) ? $formData['finish'] : null;
        $userId = Auth::User()->id;
        if (!$start || !$finish || !$userId) {
            return [self::ERROR => self::ERROR];
        } else if (Carbon::parse($start)->gte(Carbon::parse($finish))) {
            return [self::ERROR => self::ERROR];
        } else {
            $newStart = Carbon::parse($start)->format('Y-m-d H:i:s');
            $newFinish = Carbon::parse($finish)->format('Y-m-d H:i:s');

            $registerStates = RegisterState::where([['user_id', $userId], ['state', '<>', RegisterState::REGISTER_STATE_KO]])->get();

            foreach($registerStates as $registerState) {

                $company = ($registerState->myCompany != null) ? $registerState->myCompany : null;
                if ($company) {
                    $company = $company->name;
                }

                $registerStateId = $registerState->id;

                $state = $registerState->state;
                $stateName = $this->getStateName($state);
                $date = Carbon::parse($registerState->created_at)->format('d/m/Y H:i:s');

                //Buscar los otros registros
                $registers = Register::where([['register_state_id', $registerStateId],['date', '>=', $newStart], ['date', '<=', $newFinish]])->get();
                $start = '';
                $stop = '';
                $return = '';
                $finish = '';
                $typeName = '';
                $startDateTimeSelected = null;
                $finishDateTimeSelected = null;
                foreach($registers as $register) {
                    if ($register->type ==  Register::REGISTER_TYPE_START) {
                        $start = Carbon::parse($register->hour)->format('d/m/Y H:i:s');
                    }
                    if ($register->type ==  Register::REGISTER_TYPE_STOP) {
                        $stop = Carbon::parse($register->hour)->format('d/m/Y H:i:s');
                    }
                    if ($register->type ==  Register::REGISTER_TYPE_RETURN) {
                        $return = Carbon::parse($register->hour)->format('d/m/Y H:i:s');
                    }
                    if ($register->type ==  Register::REGISTER_TYPE_FINISH) {
                        $finish = Carbon::parse($register->hour)->format('d/m/Y H:i:s');
                    }
                    if ($register->type ==  Register::REGISTER_TYPE_HOLIDAYS) {
                        $startDateTime= Carbon::parse($register->date);
                        $finishDateTime = Carbon::parse($register->date);
                        if (($startDateTimeSelected == null) || $startDateTime->lt($startDateTimeSelected)) {
                            $startDateTimeSelected = $startDateTime;
                            $start = $startDateTimeSelected->format('d/m/Y');
                        }
                        if (($finishDateTimeSelected == null) || $finishDateTime->gt($finishDateTimeSelected)) {
                            $finishDateTimeSelected = $finishDateTime;
                            $finish = $finishDateTimeSelected->format('d/m/Y');
                        }

                        $type = $register->type;
                        $typeName = $this->getTypeName($type);
                    }
                }

                $dataCollection[]  = [
                    'date'       => $date,
                    'start'      => $start,
                    'stop'       => $stop,
                    'return'     => $return,
                    'finish'     => $finish,
                    'status'     => $stateName,
                    'type'       => $typeName,
                    'company'    => $company,
                ];
            }
            return [self::SUCCESS => self::SUCCESS, 'data' => $dataCollection];
        }

        return [self::SUCCESS => self::SUCCESS];
    }
}
