<?php

namespace App\Services;

use Illuminate\Contracts\Validation\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use App\Events\Team\TeamUpdated;
use App\Events\Team\TeamCreated;
use App\Events\Team\TeamDeleted;
use App\Events\Team\TeamDestroyed;
use App\Models\User;
use App\Models\Team;
use Illuminate\Support\Facades\Log;
use App\Http\Requests\Backend\TeamRequest;
use App\Exceptions\GeneralException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\ValidationException;
use Image;
use Exception;
use Throwable;
use App\Services\HelperService;
use Illuminate\Support\Facades\Session;

/**
 * Class TeamService.
 */
class TeamService
{
/**
     * @param Team $team
     *
     * @return bool
     */
    public function delete(Team $team): bool
    {
        try {
            DB::beginTransaction();

            $this->updateRelationTeam($team);
            //Logs
            Log::info('Borrado el equipo '.$team->id);
            event(new TeamDeleted($team));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

        /**
     * @param Team $team
     *
     * @return bool
     */
    public function deleteImage(Team $team): bool
    {
        try {
            DB::beginTransaction();
            $team->image_location=NULL;
            $team->Save();
            //Logs
            Log::info('Borrada la imagen '.$team->id);
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

    /**
     * @param Team $team
     *
     * @return bool
     */
    public function destroy(Team $team): bool
    {
        try {
            DB::beginTransaction();
            $this->delete($team);
            $team->forceDelete();
            //Logs
            Log::info('Destruida el equipo '.$team->id);
            event(new TeamDestroyed($team));
            DB::commit();
            return true;
        } catch (Exception $e) {
            Log::error($e);
            DB::rollBack();
        }
        return false;
    }

             /**
     * @param  TeamRequest $request
     *
     * @return Team
     * @throws GeneralException
     * @throws Throwable
     */
    public function updateTeam(TeamRequest $request): ?Team
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            //Buscar el equipo
            $team = null;
            $teamId = isset($data['team-id']) ? $data['team-id'] : null;
            if ($teamId) {
                $team = Team::where('id', $teamId)->first();
                if ($team) {
                    $team = $this->manageTeam($request, $team, $data);
                    $office = $this->updateRelationTeam($team, true);
                }
            }
            if (!$team) {
                throw new GeneralException(__('auth.general_error'));
            }
            //Logs
            Log::info('Actualizado el equipo '.$team->id);
            event(new TeamUpdated($team));
            DB::commit();
            return $team;
        } catch (ValidationException $e) {
            throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }


     /**
     * @param  TeamRequest $request
     *
     * @return Team
     * @throws GeneralException
     * @throws Throwable
     */
    public function createTeam(TeamRequest $request): ?Team
    {
        try {
            DB::beginTransaction();
            //Insertar datos
            $data = $request->all();
            $team = new Team();
            $team = $this->manageTeam($request, $team, $data);
            //Logs
            Log::info('Creado el equipo '.$team->id);
            event(new TeamCreated($team));
            DB::commit();
            return $team;
            //return false;
        } catch (ValidationException $e) {
                throw new ValidationException($e->validator);
        } catch (Exception $e) {
            Log::error($e);
            throw new GeneralException(__('auth.general_error'));
            DB::rollBack();
        }
        return null;
    }



    /**
     * @param TeamRequest $request
     * @param Team $team
     *
     * @param $data
     * @return Team
     * @throws ValidationException
     */
    private function manageTeam($request, $team, $data) {

        $team->name = isset($data['name']) ? $data['name'] : null;
        $team->slug = isset($data['name']) ? str_slug($data['name'], "-") : null;
        $team->office_id = isset($data['office']) ? $data['office'] : null;
        $team->active = isset($data['active']) ? $data['active'] : false;

        $team->Save();
        $teamId = $team->id;
        //Insertar avatar
        if ($request->hasFile('avatar')) {
            $file = $request->file('avatar');
            $filename = HelperService::uploadFile($file, 'tavatar', $teamId);
            $team->image_location = $filename;
        }
        $team->Save();
        return $team;
    }

    /**
     * @param $team
     * @param false $update
     * @return mixed
     */
    private function updateRelationTeam($team, $update = false){
        //Borrar usuarios
        $users = $team->users;
        foreach($users as $users) {
            if ($update) {
                $users->team_id = $team->id;
                User::where('team_id', '=', $team->id)->update(['team_id' => $team->id]);
                $users->Save();
            } else {
                $users->team_id = null;
                User::where('team_id', '=', $team->id)->update(['team_id' => null]);
                $users->Save();
            }
        }

        $masters = $team->masters;
        if ($update) {
            if (count($team->getChanges())) {
                $changes = $team->getChanges();
                if (isset($changes['office_id'])) {
                    Session::flash('flash_message', trans('backend.team.message.update'));
                    foreach($masters as $master) {
                    //Borrar las relaciones con usuarios
                        $team->masters()->delete();
                        $team->masters()->detach();
                    }
                }
            }
        } else {
            foreach($masters as $master) {
            //Borrar las relaciones con usuarios
            $team->masters()->delete();
            $team->masters()->detach();
            }

            $team->delete();
            $team->Save();
        }
        return $team;
    }
}
